import { useEffect, useState, createContext, useContext } from 'react';
import { SupabaseClient } from '../Util';
const AuthContext = createContext();

const AuthProvider = ({ children }) => {
  const [user, setUser] = useState();
  const [authLoading, setAuthLoading] = useState(true);

  useEffect(() => {
    const session = SupabaseClient.auth.session();
    setUser(session?.user ?? null);
    const { data: authListener } = SupabaseClient.auth.onAuthStateChange(
      (_event, session) => {
        setUser(session?.user ?? null);
      }
    );

    authListener && setAuthLoading(false);
    return () => {
      authListener?.unsubscribe();
      // console.log('unloading auth listener');
    };
  }, []);

  const signUp = async (email, password, username) => {
    const { user, session, error } = await SupabaseClient.auth.signUp({
      email,
      password,
    });
    if (error) return { error };
    return { user, session };
  };

  const signIn = async (email, password) => {
    const { user, session, error } = await SupabaseClient.auth.signIn({
      email,
      password,
    });
    if (error) return { error };
    return { user, session };
  };

  const signOut = async () => {
    const { error } = await SupabaseClient.auth.signOut();
    if (error) return error.message;
    return { status: 'success' };
  };

  const updateUserMetaData = async (newMetaData) => {
    const { error } = await SupabaseClient.auth.update({
      data: { ...newMetaData },
    });

    if (error) return error.message;
    return { status: 'success' };
  };

  const value = {
    user,
    signUp,
    signIn,
    signOut,
    updateUserMetaData,
  };

  return (
    <AuthContext.Provider value={value}>
      {!authLoading && children}
    </AuthContext.Provider>
  );
};

const useAuth = () => {
  const context = useContext(AuthContext);
  if (!context) throw new Error('useAuth must be used within AuthProvider');
  return context;
};

export { AuthProvider, useAuth };
